@@ -1,4 +1,3 @@
-#!/usr/bin/python2
 # coding=utf-8
 #
 # The Qubes OS Project, http://www.qubes-os.org
@@ -90,7 +89,7 @@ class ManageVM(object):
                 '--id={}'.format(self.vm.name), '--output=yaml',
                 'pillar.items'], stdout=subprocess.PIPE)
         (pillar_items_output, _) = p.communicate()
-        pillar_data = yaml.safe_load(pillar_items_output)
+        pillar_data = yaml.safe_load(pillar_items_output.decode())
         pillar_data = pillar_data['local']
         # remove source pillar files
         # TODO: remove also pillar modules
@@ -164,7 +163,8 @@ class ManageVM(object):
                     "Failed to copy Salt configuration to {}".
                     format(dispvm.name))
             p = dispvm.run_service('qubes.SaltLinuxVM')
-            (untrusted_stdout, _) = p.communicate(self.vm.name + '\n' + command + '\n')
+            (untrusted_stdout, _) = p.communicate(
+                    (self.vm.name + '\n' + command + '\n').encode())
             untrusted_stdout = untrusted_stdout.decode('ascii', errors='ignore')
             if not self.force_color:
                 # removing control characters, unless colors are enabled
@@ -176,17 +176,10 @@ class ManageVM(object):
 
             for line in stdout_lines:
                 self.log.info('output: %s', line)
-            if stdout_lines[0].count(self.vm.name + ':') == 1:
-                stdout_lines = stdout_lines[1:]
             self.log.info('exit code: %d', p.returncode)
-            exit_code = p.returncode
-            if exit_code == 0 and command.startswith('state.') and not stdout_lines:
-                # heuristic to detect silent failures - salt-ssh may exit with
-                # 0 in those cases
-                exit_code = 1
-                return_data = "ERROR (exit code 0, but no changes summary)"
-                self.log.info('missing changes summary, considering it a failure')
-            elif return_output and stdout_lines:
+            if return_output and stdout_lines:
+                if stdout_lines[0].count(self.vm.name + ':') == 1:
+                    stdout_lines = stdout_lines[1:]
                 return_data = stdout_lines
             elif p.returncode == 127:
                 return_data = "ERROR (missing qubes-mgmt-salt-vm-connector " \
@@ -198,6 +191,7 @@ class ManageVM(object):
                 return_data = "OK" if p.returncode == 0 else \
                     "ERROR (exit code {}, details in {})".format(
                         p.returncode,     self.log_path)
+            exit_code = p.returncode
             if self.vm.is_running() and not initially_running:
                 self.vm.shutdown()
                 # FIXME: convert to self.vm.shutdown(wait=True) in core3
@@ -306,7 +300,7 @@ def qrexec_policy(src, dst, allow):
                     policy_rules.remove(line)
 
                 with tempfile.NamedTemporaryFile(
-                        prefix=path, delete=False) as policy_new:
+                        prefix=path, delete=False, mode='w+') as policy_new:
                     policy_new.write(''.join(policy_rules))
                     policy_new.flush()
                     try:
